/**
 * This module applies all patches from the provided API endpoint to
 * build the current complete series of match events. It then exposes
 * those events as a Highland stream.
 *
 * The final stream of events is held in memory, not requested lazily
 * from the server. It does not subscribe to 'live' future events via
 * websockets. This is suitable for building a snapshot of the current
 * state of a fixture, e.g. in a stateless AWS Lambda function.
 * 
 * This module should work in the browser but assumes that
 * window.jQuery is present.
 * 
 * - window.jQuery
 */

var H = require('highland');
var MemoryCache = require('roboblogger-particle-assembler/lib/memory-cache');


if (process.env.BROWSER) {
    // use jquery in the browser
    function patchEvents(url) {
	return H(window.jQuery.getJSON(url)).pluck('events').sequence();
    }
}
else {
    // use request and streaming json parser on the server
    function patchEvents(url) {
	var request = require('request');
	var JSONStream = require('JSONStream');
	return H(request({url: url}).pipe(JSONStream.parse('events.*')));
    }
}

module.exports = function (url, /*optional*/cache) {
    cache = cache || new MemoryCache();
    return patchEvents(url).reduce(cache, function (cache, x) {
        var data = x.event;
        switch (data.type) {
        case 'append':
            cache.append(data.hash, data.event);
            break;
        case 'insertBefore':
            cache.insertBefore(data.before, data.hash, data.event);
            break;
        case 'replace':
            cache.replace(data.old, data.hash);
            break;
        case 'remove':
            cache.remove(data.hash);
            break;
        }
	cache.head = x.sequenceId;
	return cache;
    }).flatMap(function (cache) {
	return H(cache.toArray()).map(H.extend({head: cache.head}));
    });
};
